#!/usr/bin/perl
# Generated automatically from encode.pl.in by configure.

use lib '/usr/share/news-peruser';
use Peruser;

# This script uses uuenview to encode a file using either base64 or
# quoted-printable encoding, and combine it with an existing original
# message to create a MIME message of Content-type: multipart/mixed.

# Arguments: fully-qualified filename.
# Returns: 1 on success, 0 on failure.

sub encode_file
{
   my $filename = shift;

   return 0 if ( $filename eq '(cancelled)' );

   unless ( -e $filename )
   {
      feedback( "$filename does not exist!" );
      return 0;
   }

   unless ( open CHILD, "cat $home/.peruser_spool/.current_article | " .
         "uuenview -b -a $filename |" )
   {
      feedback( "Cannot fork uuenview: $!\n" );
      return 0;
   }

   unless ( safe_open( \*OUT, ">$home/.peruser4-temp" ))
   {
      return 0;
   }

   while( <CHILD> )
   {
      print OUT $_;
   }

   close CHILD;
   close OUT;

   replace_article( $server, $group, $offset, $size );

   unlink "$home/.peruser_spool/.current_article";
   unlink "$home/.peruser_spool/.current_article_origin";
   unlink "$home/.peruser4-temp";

   unless( safe_open( \*PID, "$home/.npsummary:pid" ))
   {
      feedback( "The text viewer must be running and loaded\n" .
                "with an original message, first.\n" );
      return 0;
   }

   $pid = <PID>;
   close PID;

   unless( kill( 'USR1', $pid ))
   {
      feedback( "$pid could not be signalled.\nRemoving stale " .
            "$home/.npsummary:pid\n" );
      unlink "$home/.npsummary:pid";
      return 0;
   }

   1;
}

# Replaces current article loaded in nptext with ~/.peruser4-temp
#
# Arguments: news server hostname, group name, offset and size of
#            currently-loaded article.  
# Returns: 1 on success, 0 on failure

sub replace_article
{
   my ( $server, $group, $offset, $size ) = @_;

   unless( safe_open( \*SPOOL, "$spooldir/$server-$group" ))
   {
      return 0;
   }

   unless( safe_open( \*TEMP, ">$spooldir/$server-$group:tmp" ))
   {
      close SPOOL;
      return 0;
   }

   if ( $offset )
   {
      while( <SPOOL> )
      {
         print TEMP $_;
         my $place = tell SPOOL;
         last if ( $_ eq ".\r\n" && $place == $offset );
      }
   }

   my $data;

   unless(( read SPOOL, $data, $size ) == $size )
   {
      close SPOOL;
      close TEMP;
      feedback( "Cannot read $size bytes from SPOOL: $!" );
      return 0;
   }

   undef $data;

   my $path = "$home/.peruser4-temp";
   unless( safe_open( \*NEW, $path ))
   {
      close SPOOL;
      close TEMP;
      return 0;
   }

   while( <NEW> )
   {
      chop $_;
      $_ .= "\r\n";
         
      print TEMP $_;
   }

   close NEW;
   unlink $path;

   print TEMP ".\r\n";

   while( <SPOOL> )
   {
      print TEMP $_;
   }

   close SPOOL;
   close TEMP;

   rename "$spooldir/$server-$group:tmp", "$spooldir/$server-$group";

   1;
}

# Sends SIGUSR2 to nptext to get it to dump currently-loaded article to
# ~/.peruser_spool/.current_article. called by encode_file().
#
# Arguments: none;
# Returns: 1 on success, 0 on failure

sub save_current_article
{
   unless ( safe_open( \*PID, "$home/.nptext:pid" ))
   {
      feedback( "The text viewer must be running and loaded\n" .
                "with an original message, first." );
      return 0;
   }

   my $pid = <PID>;
   safe_close( \*PID );

   chop $pid;
   my $path = "$home/.nptext:pid";

   unless( kill( 'USR2', $pid ))
   {
      feedback( "$pid could not be signalled.\nRemoving " .
            "stale $path" );
      unlink "$path";
      return 0;
   }

   unless ( safe_open( \*FILE, "$spooldir/.current_article_origin" ))
   {
      return 0;
   }

   my $success = 0;

# Globals!

   $server = <FILE>;
   $group = <FILE>;
   $offset = <FILE>;
   $size = <FILE>;

   close FILE;

   chop $server;
   chop $group;
   chop $offset;
   chop $size;

   if ( not( $group eq 'FOLLOW-UPS' || $group eq 'OUTBOX' ))
   {
      feedback( "You can only attach files to messages you have composed." );
      return 0;
   }

   return 0 if ( not select_article( $server, $group, undef, $offset, $size ));
   return 0 if ( not export_current_article( 0 ));

   1;
}

if ( save_current_article() )
{
   $filename = (( @ARGV < 1 ) ? `npgetfile` : $ARGV[ 0 ] );
   encode_file( $filename );
}
