#!/usr/bin/perl
# Generated automatically from filter.pl.in by configure.

use lib '/usr/share/news-peruser';
use Peruser;

# This script reads a file associating regular expressions to folder names,
# then attempts to copy all of the articles in all newsgroups that match
# each regular expression into the corresponding folder.

# make_list returns hash of filter/folder pairs, read from
# ~/.peruser4-filters
#
# Arguments: none 
# Return: list on success, undef on failure

sub get_filters
{
   return undef unless( safe_open( \*FILTERS, "$home/.peruser4-filters" ));

   my %filters;

   while( <FILTERS> )
   {
      chop;

      s/\\:/&/g;

      my @list = split /:/;
      $list[ 0 ] =~ s/&/:/g;
      $list[ 1 ] =~ s/&/:/g;

      $filters{ $list[ 0 ] } = $list[ 1 ];
   }

   safe_close( \*FILTERS );

   %filters;
}

# Makes an array of message-ids of all the articles current in a
# specified folder.
#
# Arguments: folder name 
# Returns: list on success, undef on failure

sub get_ids
{
   my $folder = shift;

   return undef if ( not -e "$spooldir/Folders-$folder" );
   return undef unless( safe_open( \*FOLDERS, "$spooldir/Folders-$folder"));

   my @message_ids;

   while( <FOLDERS> )
   {
      if ( /^Message-ID: +<([^>]+)>/i )
      {
         push @message_ids, $1;
      }
   }

   safe_close( \*FOLDERS );

   return @message_ids;
}

# Attempts to connect to news server and download the full article text 
# of the article with the specified Message-ID.
#
# Arguments: server hostname, group name, message_id
# Returns: non-zero on success, zero on failure.

sub get_article_text
{
   my ( $server, $group, $id ) = @_;

   if ( $connected == 0 || ( not ( $current_server eq $server )))
   {
      disconnect();
      return 0 unless( open_server( $server ));
   }

   print Peruser::SOCKET "group $group\r\n";
   my $response = <Peruser::SOCKET>;

   if ( not( substr( $response, 0, 3 ) eq '211' ))
   {
      feedback( "$current_server responded to \"group $group\\r\\n\" with " . 
         "$response\n" );
      return 0;
   }

   print Peruser::SOCKET "article $id\r\n";
   $response = <Peruser::SOCKET>;

   if ( not( substr( $response, 0, 2 ) eq '22' ))
   {
      feedback( "$current_server responded to \"article $id\\r\\n\" with " .
            "$response\n" );
      return 0;
   }

   my $line;

   do
   {
      $line = <Peruser::SOCKET>;
      print FOLDER $line;
   }
   while( not ( $line eq ".\r\n" ));

   1;
}

# Searches all spoolfiles for header-lines matching on a specified filter.
# 
# Arguments: filter, folder name.
# Returns: number of articles filtered.

sub apply_filter
{
   my $filter = shift;
   my $folder = shift;

   return 0 unless( safe_open( \*FOLDERS, "$home/.peruser4-folders" ));

   my $found = 0;

   while( <FOLDERS> )
   {
      chop;
      if ( $_ eq $folder )
      {
         $found = 1;
         last;
      }
   }

   safe_close( \*FOLDERS );

   if ( not $found )
   {
      feedback( "There is no configured folder named $folder\n" );
      return 0;
   }

   my @ids = get_ids( $folder );
   my $total = @ids;

   my $count = 0;
   my @servers = get_servers();
   return 0 if ( not defined @servers );

   my $path = "$spooldir/Folders-$folder";
   my $read_path = "$path:read";

   if ( -e $path )
   {
      return 0 unless ( safe_open( \*FOLDER, ">>$path" ));
   }
   else
   {
      return 0 unless ( safe_open( \*FOLDER, ">$path" ));
   }

   if ( -e $read_path )
   {
      return 0 unless ( safe_open( \*READ, ">>$read_path" ));
   }
   else
   {
      return 0 unless ( safe_open( \*READ, ">$read_path" ));
   }

   foreach $server ( @servers )
   {
      last if ( $server eq 'Folders' );

      my @groups = get_groups( $server );
      if ( not defined @groups )
      {
         safe_close( \*FOLDER );
         safe_close( \*READ );
         return 0;
      }

      foreach $group ( @groups )
      {
         unless( safe_open( \*SPOOL, "$spooldir/$server-$group" ))
         {
            next;
         }

         my $position = tell SPOOL;

         while( <SPOOL> )
         {
            if ( $_ =~ /^\r\n$/ )
            {
               if (( $_ = <SPOOL> ) =~ /^\r\n$/ )
               {
                  do
                  {}
                  while( not ( <SPOOL>  eq ".\r\n" ));

                  $position = tell SPOOL;
                  next;
               }
            }

            if ( $_ =~ /^\.\r\n$/ )
            {
               $position = tell SPOOL;
               next;
            }

            if ( $_ =~ /$filter/x )
            {
               unless( seek SPOOL, $position, SEEK_SET )
               {
                  safe_close( \*SPOOL );
                  safe_close( \*READ );
                  safe_close( \*FOLDER );
                  feedback( "Cannot seek to $position in $spooldir/" .
                            "$server-$group: $!\n" );
                  return 0;
               }

               my $found = 0;
               my $message_id;

               while( <SPOOL> )
               {
                  if ( /^Message-ID: +<([^>]+)>/i )
                  {
                     $message_id = "<$1>";

                     foreach $id ( @ids )
                     {
                        if ( $1 eq $id )
                        {
                           $found = 1;
                           last;
                        }
                     }

                     last;
                  }
               }

               if ( $found )
               {
                  while( not( <SPOOL> eq ".\r\n" )){};
                  $position = tell SPOOL;
                  next;
               }

               unless( seek SPOOL, $position, SEEK_SET )
               {
                  safe_close( \*SPOOL );
                  safe_close( \*READ );
                  safe_close( \*FOLDER );
                  feedback( "Cannot seek to $position in $spooldir/" .
                            "$server-$group: $!\n" );
                  return 0;
               }

               ++$count;

               print READ "u\n";

               my $initial = <SPOOL>;

               if ( substr( $initial, 0, 1 ) eq '@' )
               {
                  if ( $expand && 
                       get_article_text( $server, $group, $message_id ))
                  {
                     while( not( <SPOOL> eq ".\r\n" )){};

                     $position = tell SPOOL;
                     next;
                  }
                  else
                  {
                     print FOLDER $initial;
                     print FOLDER "X-Peruser-Server: $server\r\n";
                     print FOLDER "X-Peruser-Newsgroup: $group\r\n";
                  }
               }

               print FOLDER $initial;

               do
               {
                  $line = <SPOOL>;
                  print FOLDER $line;
               }
               while( not( $line eq ".\r\n" ));

               $position = tell SPOOL;
            }
         }

         safe_close( \*SPOOL );
      }
   }

   safe_close( \*FOLDER );
   safe_close( \*READ );

   disconnect() if ( $connected );

   $count;
}

%filters = get_filters();
if ( not %filters )
{
   feedback( "No filters have been configured\n" );
   exit( 1 );
}

$expand = 1 if ( defined $ARGV[ 0 ] && $ARGV[ 0 ] eq 'expand' );

foreach $filter ( keys %filters )
{
   $top_counts{ $filter } = apply_filter( $filter, $filters{ $filter } );
   $results .= "$filter: $top_counts{ $filter } added to " .
               "$filters{ $filter }\n";
}

if ( open PID, "$home/.npcollections:pid" )
{
   $pid = <PID>;
   chop;
   if ( defined $pid )
   {
      kill( 'USR1', $pid );
   }
   safe_close( \*PID );
}

feedback( $results );
